////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  modawbfaceassistv1.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "modawbfaceassistv1.h"
#include "parametersetmanager.h"

MODAWBFACEASSISTV1_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1TargetDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)(UINT32)triggerType[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)triggerType[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceAssistV1TargetDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceAssistV1TargetDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerType", ParameterParserDataType::Enum, true, false, 2));
    }

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modFaceAssistV1ConfidenceDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)(UINT32)triggerType[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)triggerType[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceAssistV1ConfidenceDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceAssistV1ConfidenceDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerType", ParameterParserDataType::Enum, true, false, 2));
    }

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modFaceAssistV1StatThresholdDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceAssistV1StatThresholdDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceAssistV1StatThresholdDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modFaceAssistV1FaceDetectCtrlType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "detectCntThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)detectCntThreshold, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "detectCntThresForConvChange") == 0)
    {
        ParameterUtils::DoubleToString((double)detectCntThresForConvChange, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "nonDetectCntThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)nonDetectCntThreshold, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool modFaceAssistV1FaceDetectCtrlType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "detectCntThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            detectCntThreshold = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "detectCntThresForConvChange") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            detectCntThresForConvChange = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "nonDetectCntThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            nonDetectCntThreshold = (INT32)dblValue; 
        }
    }

    return __result;
}

bool modFaceAssistV1FaceDetectCtrlType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // detectCntThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("detectCntThreshold", ParameterParserDataType::Int32, (double)detectCntThreshold, 0));
    }
    else if (PARAMETER_STRCMP(__name, "detectCntThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)detectCntThreshold, 0));
    }

    // detectCntThresForConvChange
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("detectCntThresForConvChange", ParameterParserDataType::Int32, (double)detectCntThresForConvChange, 0));
    }
    else if (PARAMETER_STRCMP(__name, "detectCntThresForConvChange") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)detectCntThresForConvChange, 0));
    }

    // nonDetectCntThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("nonDetectCntThreshold", ParameterParserDataType::Int32, (double)nonDetectCntThreshold, 0));
    }
    else if (PARAMETER_STRCMP(__name, "nonDetectCntThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)nonDetectCntThreshold, 0));
    }

    return __result;
}

bool modFaceAssistV1FaceROICtrlType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ROIXScale") == 0)
    {
        ParameterUtils::DoubleToString((double)ROIXScale, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "ROIYScale") == 0)
    {
        ParameterUtils::DoubleToString((double)ROIYScale, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "FaceSelectThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)FaceSelectThreshold, __value, __size);
        __result = true;
    }

    return __result;
}

bool modFaceAssistV1FaceROICtrlType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ROIXScale") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ROIXScale = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "ROIYScale") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ROIYScale = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "FaceSelectThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            FaceSelectThreshold = (float)dblValue; 
        }
    }

    return __result;
}

bool modFaceAssistV1FaceROICtrlType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // ROIXScale
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ROIXScale", ParameterParserDataType::Float, (double)ROIXScale));
    }
    else if (PARAMETER_STRCMP(__name, "ROIXScale") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)ROIXScale));
    }

    // ROIYScale
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ROIYScale", ParameterParserDataType::Float, (double)ROIYScale));
    }
    else if (PARAMETER_STRCMP(__name, "ROIYScale") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)ROIYScale));
    }

    // FaceSelectThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("FaceSelectThreshold", ParameterParserDataType::Float, (double)FaceSelectThreshold));
    }
    else if (PARAMETER_STRCMP(__name, "FaceSelectThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)FaceSelectThreshold));
    }

    return __result;
}

bool modFaceAssistV1CbCrDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)(UINT32)triggerType[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)triggerType[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceAssistV1CbCrDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceAssistV1CbCrDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerType", ParameterParserDataType::Enum, true, false, 2));
    }

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modFaceAssistV1CbCrDiffDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)(UINT32)triggerType[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)triggerType[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceAssistV1CbCrDiffDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceAssistV1CbCrDiffDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerType", ParameterParserDataType::Enum, true, false, 2));
    }

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modFaceAssistV1OutlierDistType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)(UINT32)triggerType[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)triggerType[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceAssistV1OutlierDistType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    triggerType[__index] = (awbFaceSATriggerCtrlType)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceAssistV1OutlierDistType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerType", ParameterParserDataType::Enum, true, false, 2));
    }

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modFaceAssistV1AWBGainLockCtrlType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "AWBGainLockEnable") == 0)
    {
        ParameterUtils::DoubleToString((double)AWBGainLockEnable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "RGainPctDev") == 0)
    {
        ParameterUtils::DoubleToString((double)RGainPctDev, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "BGainPctDev") == 0)
    {
        ParameterUtils::DoubleToString((double)BGainPctDev, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainStableCount") == 0)
    {
        ParameterUtils::DoubleToString((double)AWBGainStableCount, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockCount") == 0)
    {
        ParameterUtils::DoubleToString((double)AWBGainLockCount, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool modFaceAssistV1AWBGainLockCtrlType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "AWBGainLockEnable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            AWBGainLockEnable = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "RGainPctDev") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            RGainPctDev = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "BGainPctDev") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            BGainPctDev = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainStableCount") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            AWBGainStableCount = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockCount") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            AWBGainLockCount = (INT32)dblValue; 
        }
    }

    return __result;
}

bool modFaceAssistV1AWBGainLockCtrlType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // AWBGainLockEnable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("AWBGainLockEnable", ParameterParserDataType::Int32, (double)AWBGainLockEnable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockEnable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)AWBGainLockEnable, 0));
    }

    // RGainPctDev
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("RGainPctDev", ParameterParserDataType::Float, (double)RGainPctDev));
    }
    else if (PARAMETER_STRCMP(__name, "RGainPctDev") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)RGainPctDev));
    }

    // BGainPctDev
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("BGainPctDev", ParameterParserDataType::Float, (double)BGainPctDev));
    }
    else if (PARAMETER_STRCMP(__name, "BGainPctDev") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)BGainPctDev));
    }

    // AWBGainStableCount
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("AWBGainStableCount", ParameterParserDataType::Int32, (double)AWBGainStableCount, 0));
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainStableCount") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)AWBGainStableCount, 0));
    }

    // AWBGainLockCount
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("AWBGainLockCount", ParameterParserDataType::Int32, (double)AWBGainLockCount, 0));
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockCount") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)AWBGainLockCount, 0));
    }

    return __result;
}

bool modFaceAssistV1SkinStatsScreeningType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "StatsOutlierDistanceFilterEnable") == 0)
    {
        ParameterUtils::DoubleToString((double)StatsOutlierDistanceFilterEnable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "LockDecisionOutlierDistanceFilterEnable") == 0)
    {
        ParameterUtils::DoubleToString((double)LockDecisionOutlierDistanceFilterEnable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "StatsOutlierDistanceScreening") == 0)
    {
        __result = StatsOutlierDistanceScreening.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "lockDecisionPointOutlierDistanceScreening") == 0)
    {
        __result = lockDecisionPointOutlierDistanceScreening.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "ChromaFilterEnable") == 0)
    {
        ParameterUtils::DoubleToString((double)ChromaFilterEnable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "CrCbDiffForChromaFilter") == 0)
    {
        __result = CrCbDiffForChromaFilter.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool modFaceAssistV1SkinStatsScreeningType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "StatsOutlierDistanceFilterEnable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            StatsOutlierDistanceFilterEnable = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "LockDecisionOutlierDistanceFilterEnable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            LockDecisionOutlierDistanceFilterEnable = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "StatsOutlierDistanceScreening") == 0)
    {
        __result = StatsOutlierDistanceScreening.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "lockDecisionPointOutlierDistanceScreening") == 0)
    {
        __result = lockDecisionPointOutlierDistanceScreening.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "ChromaFilterEnable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ChromaFilterEnable = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "CrCbDiffForChromaFilter") == 0)
    {
        __result = CrCbDiffForChromaFilter.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool modFaceAssistV1SkinStatsScreeningType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // StatsOutlierDistanceFilterEnable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("StatsOutlierDistanceFilterEnable", ParameterParserDataType::Int32, (double)StatsOutlierDistanceFilterEnable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "StatsOutlierDistanceFilterEnable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)StatsOutlierDistanceFilterEnable, 0));
    }

    // LockDecisionOutlierDistanceFilterEnable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("LockDecisionOutlierDistanceFilterEnable", ParameterParserDataType::Int32, (double)LockDecisionOutlierDistanceFilterEnable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "LockDecisionOutlierDistanceFilterEnable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)LockDecisionOutlierDistanceFilterEnable, 0));
    }

    // StatsOutlierDistanceScreening
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("StatsOutlierDistanceScreening", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "StatsOutlierDistanceScreening") == 0)
    {
        __result = StatsOutlierDistanceScreening.GetInfo(&__path[__pos], __info);
    }

    // lockDecisionPointOutlierDistanceScreening
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("lockDecisionPointOutlierDistanceScreening", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "lockDecisionPointOutlierDistanceScreening") == 0)
    {
        __result = lockDecisionPointOutlierDistanceScreening.GetInfo(&__path[__pos], __info);
    }

    // ChromaFilterEnable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ChromaFilterEnable", ParameterParserDataType::Int32, (double)ChromaFilterEnable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "ChromaFilterEnable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)ChromaFilterEnable, 0));
    }

    // CrCbDiffForChromaFilter
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("CrCbDiffForChromaFilter", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "CrCbDiffForChromaFilter") == 0)
    {
        __result = CrCbDiffForChromaFilter.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool modFaceSARefIlluConfidenceDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            ParameterUtils::DoubleToString((double)(UINT32)triggerType[__index], __value, __size, 0);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < 2; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)(UINT32)triggerType[i], &__value[__valuePos], (INT32)(__size - __valuePos), 0);
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modFaceSARefIlluConfidenceDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerType") == 0)
    {
        if (__index < 2)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                triggerType[__index] = (awbFaceSARefIlluTriggerCtrlType)(UINT32)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < 2)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    triggerType[__index] = (awbFaceSARefIlluTriggerCtrlType)(UINT32)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modFaceSARefIlluConfidenceDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerType", ParameterParserDataType::Enum, true, false, 2));
    }

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modReferenceCCTType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "CCT") == 0)
    {
        if (__index < CCTCount)
        {
            ParameterUtils::DoubleToString((double)CCT[__index], __value, __size);
            __result = true;
        }
        else
        {
            SIZE_T __valuePos = 0;
            for (UINT32 i = 0; i < CCTCount; i++)
            {
                if (i > 0)
                {
                    PARAMETER_STRCAT(__value, __size, ",");
                    __valuePos = PARAMETER_STRLEN(__value);
                }
                ParameterUtils::DoubleToString((double)CCT[i], &__value[__valuePos], (INT32)(__size - __valuePos));
            }
            __result = true;
        }
    }

    return __result;
}

bool modReferenceCCTType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "CCT") == 0)
    {
        if (__index < CCTCount)
        {
            const double dblValue = ParameterUtils::StringToDouble(__value, __result);
            if (__result)
            {
                CCT[__index] = (float)dblValue; 
            }
        }
        else
        {
            SIZE_T __valueLen = PARAMETER_STRLEN(__value);
            UINT32 __pos = 0;

            __index = 0;
            while (__pos < __valueLen && __index < CCTCount)
            {
                CHAR __valueStr[MAX_SIZE];

                __pos += ParameterUtils::GetSubString(&__value[__pos], ',', __valueStr, MAX_SIZE);
                __pos++;

                const double dblValue = ParameterUtils::StringToDouble(__valueStr, __result);
                if (__result)
                {
                    CCT[__index] = (float)dblValue; 
                }
                __index++;
            }
            __result = true;
        }
    }

    return __result;
}

bool modReferenceCCTType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // CCT
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("CCT", ParameterParserDataType::Float, true, false, CCTCount));
    }

    return __result;
}

bool modRefIlluCtrlType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "refIlluEnable") == 0)
    {
        ParameterUtils::DoubleToString((double)refIlluEnable, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "referenceCCTs") == 0)
    {
        if (__index < referenceCCTsCount)
        {
            __result = referenceCCTs[__index].GetValue(&__path[__pos], __value, __size);
        }
    }
    else if (PARAMETER_STRCMP(__name, "activateExceedFaceStatsCntPercentage") == 0)
    {
        ParameterUtils::DoubleToString((double)activateExceedFaceStatsCntPercentage, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dynamicSkipFrameCnt") == 0)
    {
        ParameterUtils::DoubleToString((double)dynamicSkipFrameCnt, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "refIlluConfidenceLevel") == 0)
    {
        if (__index < refIlluConfidenceLevelCount)
        {
            __result = refIlluConfidenceLevel[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modRefIlluCtrlType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "refIlluEnable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            refIlluEnable = (awbglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "referenceCCTs") == 0)
    {
        if (__index < referenceCCTsCount)
        {
            __result = referenceCCTs[__index].SetValue(&__path[__pos], __value);
        }
    }
    else if (PARAMETER_STRCMP(__name, "activateExceedFaceStatsCntPercentage") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            activateExceedFaceStatsCntPercentage = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dynamicSkipFrameCnt") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dynamicSkipFrameCnt = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "refIlluConfidenceLevel") == 0)
    {
        if (__index < refIlluConfidenceLevelCount)
        {
            __result = refIlluConfidenceLevel[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modRefIlluCtrlType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // refIlluEnable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("refIlluEnable", ParameterParserDataType::Int32, (double)refIlluEnable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "refIlluEnable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)refIlluEnable, 0));
    }

    // referenceCCTs
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("referenceCCTs", ParameterParserDataType::Structure, true, false, referenceCCTsCount));
    }
    else if (PARAMETER_STRCMP(__name, "referenceCCTs") == 0)
    {
        if (__index < referenceCCTsCount)
        {
            __result = referenceCCTs[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, referenceCCTsCount));
        }
    }

    // activateExceedFaceStatsCntPercentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("activateExceedFaceStatsCntPercentage", ParameterParserDataType::Float, (double)activateExceedFaceStatsCntPercentage));
    }
    else if (PARAMETER_STRCMP(__name, "activateExceedFaceStatsCntPercentage") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)activateExceedFaceStatsCntPercentage));
    }

    // dynamicSkipFrameCnt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dynamicSkipFrameCnt", ParameterParserDataType::Int32, (double)dynamicSkipFrameCnt, 0));
    }
    else if (PARAMETER_STRCMP(__name, "dynamicSkipFrameCnt") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)dynamicSkipFrameCnt, 0));
    }

    // refIlluConfidenceLevel
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("refIlluConfidenceLevel", ParameterParserDataType::Structure, true, false, refIlluConfidenceLevelCount));
    }
    else if (PARAMETER_STRCMP(__name, "refIlluConfidenceLevel") == 0)
    {
        if (__index < refIlluConfidenceLevelCount)
        {
            __result = refIlluConfidenceLevel[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, refIlluConfidenceLevelCount));
        }
    }

    return __result;
}

bool modSubRgnChromaDiffThDataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool modSubRgnChromaDiffThDataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool modSubRgnChromaDiffThDataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // triggerPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "triggerPt") == 0)
    {
        __result = triggerPt.GetInfo(&__path[__pos], __info);
    }

    // triggerData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("triggerData", ParameterParserDataType::Structure, true, false, triggerDataCount));
    }
    else if (PARAMETER_STRCMP(__name, "triggerData") == 0)
    {
        if (__index < triggerDataCount)
        {
            __result = triggerData[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, triggerDataCount));
        }
    }

    return __result;
}

bool modSubRgnChormaAssistType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "chromaDiffThreshold") == 0)
    {
        __result = chromaDiffThreshold.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "AECFaceROIUpdate") == 0)
    {
        __result = AECFaceROIUpdate.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "referenceIlluminant") == 0)
    {
        __result = referenceIlluminant.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool modSubRgnChormaAssistType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "chromaDiffThreshold") == 0)
    {
        __result = chromaDiffThreshold.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "AECFaceROIUpdate") == 0)
    {
        __result = AECFaceROIUpdate.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "referenceIlluminant") == 0)
    {
        __result = referenceIlluminant.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool modSubRgnChormaAssistType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // chromaDiffThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromaDiffThreshold", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromaDiffThreshold") == 0)
    {
        __result = chromaDiffThreshold.GetInfo(&__path[__pos], __info);
    }

    // AECFaceROIUpdate
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("AECFaceROIUpdate", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "AECFaceROIUpdate") == 0)
    {
        __result = AECFaceROIUpdate.GetInfo(&__path[__pos], __info);
    }

    // referenceIlluminant
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("referenceIlluminant", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "referenceIlluminant") == 0)
    {
        __result = referenceIlluminant.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool modSubRgnChormaAssistType::AECFaceROIUpdateStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hysteresisPerc") == 0)
    {
        ParameterUtils::DoubleToString((double)hysteresisPerc, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "hysteresisFrameCnt") == 0)
    {
        ParameterUtils::DoubleToString((double)hysteresisFrameCnt, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool modSubRgnChormaAssistType::AECFaceROIUpdateStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hysteresisPerc") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hysteresisPerc = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "hysteresisFrameCnt") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            hysteresisFrameCnt = (INT32)dblValue; 
        }
    }

    return __result;
}

bool modSubRgnChormaAssistType::AECFaceROIUpdateStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // hysteresisPerc
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hysteresisPerc", ParameterParserDataType::Float, (double)hysteresisPerc));
    }
    else if (PARAMETER_STRCMP(__name, "hysteresisPerc") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)hysteresisPerc));
    }

    // hysteresisFrameCnt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("hysteresisFrameCnt", ParameterParserDataType::Int32, (double)hysteresisFrameCnt, 0));
    }
    else if (PARAMETER_STRCMP(__name, "hysteresisFrameCnt") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)hysteresisFrameCnt, 0));
    }

    return __result;
}

bool modSubRgnChormaAssistType::referenceIlluminantStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "useCase") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)useCase, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool modSubRgnChormaAssistType::referenceIlluminantStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "useCase") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            useCase = (refIlluSubRgnUseCaseType)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool modSubRgnChormaAssistType::referenceIlluminantStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // useCase
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("useCase", ParameterParserDataType::Enum, (double)(UINT32)useCase, 0));
    }
    else if (PARAMETER_STRCMP(__name, "useCase") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)useCase, 0));
    }

    return __result;
}

bool modContourFilterType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool modContourFilterType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbglobalelements::enableFlagType)dblValue; 
        }
    }

    return __result;
}

bool modContourFilterType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    return __result;
}

bool modFaceDecisionTree::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "ROISizeRatioThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)ROISizeRatioThreshold, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "eyeAngleDiffThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)eyeAngleDiffThreshold, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "faceAngleDiffThreshold") == 0)
    {
        ParameterUtils::DoubleToString((double)faceAngleDiffThreshold, __value, __size);
        __result = true;
    }

    return __result;
}

bool modFaceDecisionTree::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "ROISizeRatioThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ROISizeRatioThreshold = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "eyeAngleDiffThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            eyeAngleDiffThreshold = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "faceAngleDiffThreshold") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            faceAngleDiffThreshold = (float)dblValue; 
        }
    }

    return __result;
}

bool modFaceDecisionTree::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // ROISizeRatioThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ROISizeRatioThreshold", ParameterParserDataType::Float, (double)ROISizeRatioThreshold));
    }
    else if (PARAMETER_STRCMP(__name, "ROISizeRatioThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)ROISizeRatioThreshold));
    }

    // eyeAngleDiffThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("eyeAngleDiffThreshold", ParameterParserDataType::Float, (double)eyeAngleDiffThreshold));
    }
    else if (PARAMETER_STRCMP(__name, "eyeAngleDiffThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)eyeAngleDiffThreshold));
    }

    // faceAngleDiffThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("faceAngleDiffThreshold", ParameterParserDataType::Float, (double)faceAngleDiffThreshold));
    }
    else if (PARAMETER_STRCMP(__name, "faceAngleDiffThreshold") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)faceAngleDiffThreshold));
    }

    return __result;
}

bool chromatixAWBFaceAssistV1Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        ParameterUtils::DoubleToString((double)enableFlag, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "faceAssistSAWeight") == 0)
    {
        ParameterUtils::DoubleToString((double)faceAssistSAWeight, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "FaceROICtrl") == 0)
    {
        __result = FaceROICtrl.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "faceDetectCtrl") == 0)
    {
        __result = faceDetectCtrl.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "ConvSpeed") == 0)
    {
        ParameterUtils::DoubleToString((double)ConvSpeed, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "darkThresholdData") == 0)
    {
        __result = darkThresholdData.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "SatThresholdData") == 0)
    {
        __result = SatThresholdData.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "confidenceLevel") == 0)
    {
        if (__index < confidenceLevelCount)
        {
            __result = confidenceLevel[__index].GetValue(&__path[__pos], __value, __size);
        }
    }
    else if (PARAMETER_STRCMP(__name, "darkSkinTargetPt") == 0)
    {
        __result = darkSkinTargetPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "lightSkinTargetPt") == 0)
    {
        __result = lightSkinTargetPt.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "skinTypeInterpWeight") == 0)
    {
        if (__index < skinTypeInterpWeightCount)
        {
            __result = skinTypeInterpWeight[__index].GetValue(&__path[__pos], __value, __size);
        }
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockCtrl") == 0)
    {
        __result = AWBGainLockCtrl.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "CbCrHighThreshold") == 0)
    {
        __result = CbCrHighThreshold.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "CbCrLowThreshold") == 0)
    {
        __result = CbCrLowThreshold.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "SkinStatsScreening") == 0)
    {
        __result = SkinStatsScreening.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "RefIlluFace") == 0)
    {
        __result = RefIlluFace.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "SubRegionChormaAssist") == 0)
    {
        __result = SubRegionChormaAssist.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "ContourFilter") == 0)
    {
        __result = ContourFilter.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "FaceDecisionTree") == 0)
    {
        __result = FaceDecisionTree.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatixAWBFaceAssistV1Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableFlag = (awbglobalelements::enableFlagType)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "faceAssistSAWeight") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            faceAssistSAWeight = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "FaceROICtrl") == 0)
    {
        __result = FaceROICtrl.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "faceDetectCtrl") == 0)
    {
        __result = faceDetectCtrl.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "ConvSpeed") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ConvSpeed = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "darkThresholdData") == 0)
    {
        __result = darkThresholdData.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "SatThresholdData") == 0)
    {
        __result = SatThresholdData.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "confidenceLevel") == 0)
    {
        if (__index < confidenceLevelCount)
        {
            __result = confidenceLevel[__index].SetValue(&__path[__pos], __value);
        }
    }
    else if (PARAMETER_STRCMP(__name, "darkSkinTargetPt") == 0)
    {
        __result = darkSkinTargetPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "lightSkinTargetPt") == 0)
    {
        __result = lightSkinTargetPt.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "skinTypeInterpWeight") == 0)
    {
        if (__index < skinTypeInterpWeightCount)
        {
            __result = skinTypeInterpWeight[__index].SetValue(&__path[__pos], __value);
        }
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockCtrl") == 0)
    {
        __result = AWBGainLockCtrl.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "CbCrHighThreshold") == 0)
    {
        __result = CbCrHighThreshold.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "CbCrLowThreshold") == 0)
    {
        __result = CbCrLowThreshold.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "SkinStatsScreening") == 0)
    {
        __result = SkinStatsScreening.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "RefIlluFace") == 0)
    {
        __result = RefIlluFace.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "SubRegionChormaAssist") == 0)
    {
        __result = SubRegionChormaAssist.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "ContourFilter") == 0)
    {
        __result = ContourFilter.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "FaceDecisionTree") == 0)
    {
        __result = FaceDecisionTree.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatixAWBFaceAssistV1Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // moduleID
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("moduleID", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = moduleID.GetInfo(&__path[__pos], __info);
    }

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // enableFlag
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableFlag", ParameterParserDataType::Int32, (double)enableFlag, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableFlag") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)enableFlag, 0));
    }

    // faceAssistSAWeight
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("faceAssistSAWeight", ParameterParserDataType::Float, (double)faceAssistSAWeight));
    }
    else if (PARAMETER_STRCMP(__name, "faceAssistSAWeight") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)faceAssistSAWeight));
    }

    // FaceROICtrl
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("FaceROICtrl", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "FaceROICtrl") == 0)
    {
        __result = FaceROICtrl.GetInfo(&__path[__pos], __info);
    }

    // faceDetectCtrl
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("faceDetectCtrl", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "faceDetectCtrl") == 0)
    {
        __result = faceDetectCtrl.GetInfo(&__path[__pos], __info);
    }

    // ConvSpeed
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ConvSpeed", ParameterParserDataType::Float, (double)ConvSpeed));
    }
    else if (PARAMETER_STRCMP(__name, "ConvSpeed") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)ConvSpeed));
    }

    // darkThresholdData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("darkThresholdData", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "darkThresholdData") == 0)
    {
        __result = darkThresholdData.GetInfo(&__path[__pos], __info);
    }

    // SatThresholdData
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("SatThresholdData", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "SatThresholdData") == 0)
    {
        __result = SatThresholdData.GetInfo(&__path[__pos], __info);
    }

    // confidenceLevel
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("confidenceLevel", ParameterParserDataType::Structure, true, false, confidenceLevelCount));
    }
    else if (PARAMETER_STRCMP(__name, "confidenceLevel") == 0)
    {
        if (__index < confidenceLevelCount)
        {
            __result = confidenceLevel[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, confidenceLevelCount));
        }
    }

    // darkSkinTargetPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("darkSkinTargetPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "darkSkinTargetPt") == 0)
    {
        __result = darkSkinTargetPt.GetInfo(&__path[__pos], __info);
    }

    // lightSkinTargetPt
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("lightSkinTargetPt", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "lightSkinTargetPt") == 0)
    {
        __result = lightSkinTargetPt.GetInfo(&__path[__pos], __info);
    }

    // skinTypeInterpWeight
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("skinTypeInterpWeight", ParameterParserDataType::Structure, true, false, skinTypeInterpWeightCount));
    }
    else if (PARAMETER_STRCMP(__name, "skinTypeInterpWeight") == 0)
    {
        if (__index < skinTypeInterpWeightCount)
        {
            __result = skinTypeInterpWeight[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, skinTypeInterpWeightCount));
        }
    }

    // AWBGainLockCtrl
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("AWBGainLockCtrl", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "AWBGainLockCtrl") == 0)
    {
        __result = AWBGainLockCtrl.GetInfo(&__path[__pos], __info);
    }

    // CbCrHighThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("CbCrHighThreshold", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "CbCrHighThreshold") == 0)
    {
        __result = CbCrHighThreshold.GetInfo(&__path[__pos], __info);
    }

    // CbCrLowThreshold
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("CbCrLowThreshold", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "CbCrLowThreshold") == 0)
    {
        __result = CbCrLowThreshold.GetInfo(&__path[__pos], __info);
    }

    // SkinStatsScreening
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("SkinStatsScreening", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "SkinStatsScreening") == 0)
    {
        __result = SkinStatsScreening.GetInfo(&__path[__pos], __info);
    }

    // RefIlluFace
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("RefIlluFace", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "RefIlluFace") == 0)
    {
        __result = RefIlluFace.GetInfo(&__path[__pos], __info);
    }

    // SubRegionChormaAssist
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("SubRegionChormaAssist", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "SubRegionChormaAssist") == 0)
    {
        __result = SubRegionChormaAssist.GetInfo(&__path[__pos], __info);
    }

    // ContourFilter
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ContourFilter", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "ContourFilter") == 0)
    {
        __result = ContourFilter.GetInfo(&__path[__pos], __info);
    }

    // FaceDecisionTree
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("FaceDecisionTree", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "FaceDecisionTree") == 0)
    {
        __result = FaceDecisionTree.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatixAWBFaceAssistV1Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatixAWBFaceAssistV1Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatixAWBFaceAssistV1Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_awbFaceSATriggerCtrlType(
    ParameterFileSymbolTableEntry* const entry,
    awbFaceSATriggerCtrlType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlLuxIdx):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlLuxIdx;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlCCT):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlSgwCCT):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlSgwCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlValidStatsCntRatio):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlValidStatsCntRatio;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlFaceStatsCntRatio):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlFaceStatsCntRatio;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlLightSkinCCT):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlLightSkinCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlDarkSkinCCT):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlDarkSkinCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlSkinTypeDistRatio):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlSkinTypeDistRatio;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlAgwRg):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlAgwRg;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlAgwBg):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlAgwBg;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlFianlfaceCCT):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlFianlfaceCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlFilterStatsCntRatio):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlFilterStatsCntRatio;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlFaceChromaDiff):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlFaceChromaDiff;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSATriggerCtrlType::FaceTriggerCtrlTypeMax):
            enumValue = awbFaceSATriggerCtrlType::FaceTriggerCtrlTypeMax;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to awbFaceSATriggerCtrlType", uintValue);
            break;
        }
    }
    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_awbFaceSARefIlluTriggerCtrlType(
    ParameterFileSymbolTableEntry* const entry,
    awbFaceSARefIlluTriggerCtrlType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlLuxIdx):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlLuxIdx;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSgwCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSgwCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlAgwCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlAgwCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlRefCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlRefCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSpectralSensorCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSpectralSensorCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSpectralSensorIR):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSpectralSensorIR;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSpectralSensorIR2):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlSpectralSensorIR2;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlDiffAgwCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlDiffAgwCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlDiffSpectralSensorCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlDiffSpectralSensorCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlRefStatsCntRelRatio):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlRefStatsCntRelRatio;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlDiffSgwCCT):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlDiffSgwCCT;
            result    = true;
            break;
        case static_cast<UINT32>(awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlTypeMax):
            enumValue = awbFaceSARefIlluTriggerCtrlType::FaceRefIlluTriggerCtrlTypeMax;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to awbFaceSARefIlluTriggerCtrlType", uintValue);
            break;
        }
    }
    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_refIlluSubRgnUseCaseType(
    ParameterFileSymbolTableEntry* const entry,
    refIlluSubRgnUseCaseType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(refIlluSubRgnUseCaseType::ActivateAboveThreshold):
            enumValue = refIlluSubRgnUseCaseType::ActivateAboveThreshold;
            result    = true;
            break;
        case static_cast<UINT32>(refIlluSubRgnUseCaseType::ActivateBelowThreshold):
            enumValue = refIlluSubRgnUseCaseType::ActivateBelowThreshold;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to refIlluSubRgnUseCaseType", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatixAWBFaceAssistV1TypeClass::chromatixAWBFaceAssistV1TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(faceAssistSAWeight);
    PARAMETER_INITIALIZE(FaceROICtrl);
    PARAMETER_INITIALIZE(faceDetectCtrl);
    PARAMETER_INITIALIZE(ConvSpeed);
    PARAMETER_INITIALIZE(darkThresholdData);
    PARAMETER_INITIALIZE(SatThresholdData);
    PARAMETER_INITIALIZE(confidenceLevelCount);
    PARAMETER_INITIALIZE(confidenceLevel);
    PARAMETER_INITIALIZE(darkSkinTargetPt);
    PARAMETER_INITIALIZE(lightSkinTargetPt);
    PARAMETER_INITIALIZE(skinTypeInterpWeightCount);
    PARAMETER_INITIALIZE(skinTypeInterpWeight);
    PARAMETER_INITIALIZE(AWBGainLockCtrl);
    PARAMETER_INITIALIZE(CbCrHighThreshold);
    PARAMETER_INITIALIZE(CbCrLowThreshold);
    PARAMETER_INITIALIZE(SkinStatsScreening);
    PARAMETER_INITIALIZE(RefIlluFace);
    PARAMETER_INITIALIZE(SubRegionChormaAssist);
    PARAMETER_INITIALIZE(ContourFilter);
    PARAMETER_INITIALIZE(FaceDecisionTree);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatixAWBFaceAssistV1TypeClass::chromatixAWBFaceAssistV1TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(faceAssistSAWeight);
    PARAMETER_INITIALIZE(FaceROICtrl);
    PARAMETER_INITIALIZE(faceDetectCtrl);
    PARAMETER_INITIALIZE(ConvSpeed);
    PARAMETER_INITIALIZE(darkThresholdData);
    PARAMETER_INITIALIZE(SatThresholdData);
    PARAMETER_INITIALIZE(confidenceLevel);
    PARAMETER_INITIALIZE(darkSkinTargetPt);
    PARAMETER_INITIALIZE(lightSkinTargetPt);
    PARAMETER_INITIALIZE(skinTypeInterpWeight);
    PARAMETER_INITIALIZE(AWBGainLockCtrl);
    PARAMETER_INITIALIZE(CbCrHighThreshold);
    PARAMETER_INITIALIZE(CbCrLowThreshold);
    PARAMETER_INITIALIZE(SkinStatsScreening);
    PARAMETER_INITIALIZE(RefIlluFace);
    PARAMETER_INITIALIZE(SubRegionChormaAssist);
    PARAMETER_INITIALIZE(ContourFilter);
    PARAMETER_INITIALIZE(FaceDecisionTree);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatixAWBFaceAssistV1TypeClass::~chromatixAWBFaceAssistV1TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatixAWBFaceAssistV1TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "faceAssistV1")) == 0)
    {
        name = "faceAssistV1";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1TargetDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1TargetDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerType);
    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    for (UINT32 i = 0; result && i < 2; i++)
    {
        result = modawbfaceassistv1::ReadEnum_awbFaceSATriggerCtrlType(entry, triggerType[i], alignment);
    }
    if (result)
    {
        result = triggerPt.Load(entry, alignment);
    }

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbSATargetLevel2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbSATargetLevel2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1ConfidenceDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1ConfidenceDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerType);
    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    for (UINT32 i = 0; result && i < 2; i++)
    {
        result = modawbfaceassistv1::ReadEnum_awbFaceSATriggerCtrlType(entry, triggerType[i], alignment);
    }
    if (result)
    {
        result = triggerPt.Load(entry, alignment);
    }

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbSAConfidenceLevel2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbSAConfidenceLevel2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1StatThresholdDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1StatThresholdDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    result = triggerPt.Load(entry, alignment);

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbTriggerDataRGBType*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbTriggerDataRGBType[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1FaceDetectCtrlType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1FaceDetectCtrlType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(detectCntThreshold);
    PARAMETER_INITIALIZE(detectCntThresForConvChange);
    PARAMETER_INITIALIZE(nonDetectCntThreshold);

    SYMBOL_TABLE_ENTRY_READ(entry, result, detectCntThreshold, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, detectCntThresForConvChange, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, nonDetectCntThreshold, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1FaceROICtrlType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1FaceROICtrlType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(ROIXScale);
    PARAMETER_INITIALIZE(ROIYScale);
    PARAMETER_INITIALIZE(FaceSelectThreshold);

    SYMBOL_TABLE_ENTRY_READ(entry, result, ROIXScale, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, ROIYScale, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, FaceSelectThreshold, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1CbCrDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1CbCrDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerType);
    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    for (UINT32 i = 0; result && i < 2; i++)
    {
        result = modawbfaceassistv1::ReadEnum_awbFaceSATriggerCtrlType(entry, triggerType[i], alignment);
    }
    if (result)
    {
        result = triggerPt.Load(entry, alignment);
    }

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbCbCrLevel2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbCbCrLevel2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1CbCrDiffDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1CbCrDiffDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerType);
    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    for (UINT32 i = 0; result && i < 2; i++)
    {
        result = modawbfaceassistv1::ReadEnum_awbFaceSATriggerCtrlType(entry, triggerType[i], alignment);
    }
    if (result)
    {
        result = triggerPt.Load(entry, alignment);
    }

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbCbCrDiffLevel2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbCbCrDiffLevel2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1OutlierDistType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1OutlierDistType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerType);
    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    for (UINT32 i = 0; result && i < 2; i++)
    {
        result = modawbfaceassistv1::ReadEnum_awbFaceSATriggerCtrlType(entry, triggerType[i], alignment);
    }
    if (result)
    {
        result = triggerPt.Load(entry, alignment);
    }

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbOutlierDist2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbOutlierDist2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1AWBGainLockCtrlType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1AWBGainLockCtrlType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(AWBGainLockEnable);
    PARAMETER_INITIALIZE(RGainPctDev);
    PARAMETER_INITIALIZE(BGainPctDev);
    PARAMETER_INITIALIZE(AWBGainStableCount);
    PARAMETER_INITIALIZE(AWBGainLockCount);

    SYMBOL_TABLE_ENTRY_READ(entry, result, AWBGainLockEnable, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, RGainPctDev, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, BGainPctDev, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, AWBGainStableCount, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, AWBGainLockCount, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceAssistV1SkinStatsScreeningType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceAssistV1SkinStatsScreeningType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(StatsOutlierDistanceFilterEnable);
    PARAMETER_INITIALIZE(LockDecisionOutlierDistanceFilterEnable);
    PARAMETER_INITIALIZE(StatsOutlierDistanceScreening);
    PARAMETER_INITIALIZE(lockDecisionPointOutlierDistanceScreening);
    PARAMETER_INITIALIZE(ChromaFilterEnable);
    PARAMETER_INITIALIZE(CrCbDiffForChromaFilter);

    SYMBOL_TABLE_ENTRY_READ(entry, result, StatsOutlierDistanceFilterEnable, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, LockDecisionOutlierDistanceFilterEnable, alignment)
    if (result)
    {
        result = StatsOutlierDistanceScreening.Load(entry, alignment);
    }
    if (result)
    {
        result = lockDecisionPointOutlierDistanceScreening.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, ChromaFilterEnable, alignment)
    if (result)
    {
        result = CrCbDiffForChromaFilter.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceSARefIlluConfidenceDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceSARefIlluConfidenceDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerType);
    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    for (UINT32 i = 0; result && i < 2; i++)
    {
        result = modawbfaceassistv1::ReadEnum_awbFaceSARefIlluTriggerCtrlType(entry, triggerType[i], alignment);
    }
    if (result)
    {
        result = triggerPt.Load(entry, alignment);
    }

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbSAConfidenceLevel2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbSAConfidenceLevel2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modReferenceCCTType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modReferenceCCTType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(CCT);


    // Reading Array CCT from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        CCTCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, CCTCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            CCTID = pointerEntry->ID;
            DupObjInfo* CCTDupInfo = pointerEntry->GetDupObject(CCTCount, file, line);
            if (CCTDupInfo != nullptr && CCTDupInfo->Object != nullptr)
            {
                CCT = (float*)CCTDupInfo->Object;
                if (CCT == nullptr)
                {
                    if (CCTDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "CCT duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            CCTDupInfo->File, CCTDupInfo->Line, CCTDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "CCT duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            CCTDupInfo->Line, CCTDupInfo->RefCount);
                    }
                }
            }

            if (CCT == nullptr)
            {
                if (result && CCTCount > 0)
                {
                    CCT = pointerEntry->ReadArray<float>(CCTCount, alignment);
                    result = CCT != nullptr;
                }
                if (!result || CCT == nullptr)
                {
                    if (CCTCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate CCT\n");
                    }
                    CCTCount = 0;
                }
            }
        }
        if (CCT == nullptr)
        {
            CCTCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modRefIlluCtrlType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modRefIlluCtrlType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(refIlluEnable);
    PARAMETER_INITIALIZE(referenceCCTs);
    PARAMETER_INITIALIZE(activateExceedFaceStatsCntPercentage);
    PARAMETER_INITIALIZE(dynamicSkipFrameCnt);
    PARAMETER_INITIALIZE(refIlluConfidenceLevel);

    SYMBOL_TABLE_ENTRY_READ(entry, result, refIlluEnable, alignment)

    // Reading Array referenceCCTs from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        referenceCCTsCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, referenceCCTsCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            referenceCCTsID = pointerEntry->ID;
            DupObjInfo* referenceCCTsDupInfo = pointerEntry->GetDupObject(referenceCCTsCount, file, line);
            if (referenceCCTsDupInfo != nullptr && referenceCCTsDupInfo->Object != nullptr)
            {
                referenceCCTs = (modReferenceCCTType*)referenceCCTsDupInfo->Object;
                if (referenceCCTs == nullptr)
                {
                    if (referenceCCTsDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "referenceCCTs duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            referenceCCTsDupInfo->File, referenceCCTsDupInfo->Line, referenceCCTsDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "referenceCCTs duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            referenceCCTsDupInfo->Line, referenceCCTsDupInfo->RefCount);
                    }
                }
            }

            if (referenceCCTs == nullptr)
            {
                if (result && referenceCCTsCount > 0)
                {
                    referenceCCTs = PARAMETER_NEW modReferenceCCTType[referenceCCTsCount];
                    (void)pointerEntry->SetDupObject(referenceCCTs, referenceCCTsCount, file, line);
                }
                if (!result || referenceCCTs == nullptr)
                {
                    if (referenceCCTsCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate referenceCCTs\n");
                    }
                    referenceCCTsCount = 0;
                }
                for (UINT32 i = 0; result && i < referenceCCTsCount; i++)
                {
                    result = referenceCCTs[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (referenceCCTs == nullptr)
        {
            referenceCCTsCount = 0;
        }
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, activateExceedFaceStatsCntPercentage, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, dynamicSkipFrameCnt, alignment)

    // Reading Array refIlluConfidenceLevel from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        refIlluConfidenceLevelCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, refIlluConfidenceLevelCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            refIlluConfidenceLevelID = pointerEntry->ID;
            DupObjInfo* refIlluConfidenceLevelDupInfo = pointerEntry->GetDupObject(refIlluConfidenceLevelCount, file, line);
            if (refIlluConfidenceLevelDupInfo != nullptr && refIlluConfidenceLevelDupInfo->Object != nullptr)
            {
                refIlluConfidenceLevel = (modFaceSARefIlluConfidenceDataType*)refIlluConfidenceLevelDupInfo->Object;
                if (refIlluConfidenceLevel == nullptr)
                {
                    if (refIlluConfidenceLevelDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "refIlluConfidenceLevel duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            refIlluConfidenceLevelDupInfo->File, refIlluConfidenceLevelDupInfo->Line, refIlluConfidenceLevelDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "refIlluConfidenceLevel duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            refIlluConfidenceLevelDupInfo->Line, refIlluConfidenceLevelDupInfo->RefCount);
                    }
                }
            }

            if (refIlluConfidenceLevel == nullptr)
            {
                if (result && refIlluConfidenceLevelCount > 0)
                {
                    refIlluConfidenceLevel = PARAMETER_NEW modFaceSARefIlluConfidenceDataType[refIlluConfidenceLevelCount];
                    (void)pointerEntry->SetDupObject(refIlluConfidenceLevel, refIlluConfidenceLevelCount, file, line);
                }
                if (!result || refIlluConfidenceLevel == nullptr)
                {
                    if (refIlluConfidenceLevelCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate refIlluConfidenceLevel\n");
                    }
                    refIlluConfidenceLevelCount = 0;
                }
                for (UINT32 i = 0; result && i < refIlluConfidenceLevelCount; i++)
                {
                    result = refIlluConfidenceLevel[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (refIlluConfidenceLevel == nullptr)
        {
            refIlluConfidenceLevelCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modSubRgnChromaDiffThDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modSubRgnChromaDiffThDataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(triggerPt);
    PARAMETER_INITIALIZE(triggerData);

    result = triggerPt.Load(entry, alignment);

    // Reading Array triggerData from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        triggerDataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, triggerDataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            triggerDataID = pointerEntry->ID;
            DupObjInfo* triggerDataDupInfo = pointerEntry->GetDupObject(triggerDataCount, file, line);
            if (triggerDataDupInfo != nullptr && triggerDataDupInfo->Object != nullptr)
            {
                triggerData = (awbglobalelements::awbSAConfidenceLevel2Type*)triggerDataDupInfo->Object;
                if (triggerData == nullptr)
                {
                    if (triggerDataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            triggerDataDupInfo->File, triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "triggerData duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            triggerDataDupInfo->Line, triggerDataDupInfo->RefCount);
                    }
                }
            }

            if (triggerData == nullptr)
            {
                if (result && triggerDataCount > 0)
                {
                    triggerData = PARAMETER_NEW awbglobalelements::awbSAConfidenceLevel2Type[triggerDataCount];
                    (void)pointerEntry->SetDupObject(triggerData, triggerDataCount, file, line);
                }
                if (!result || triggerData == nullptr)
                {
                    if (triggerDataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate triggerData\n");
                    }
                    triggerDataCount = 0;
                }
                for (UINT32 i = 0; result && i < triggerDataCount; i++)
                {
                    result = triggerData[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (triggerData == nullptr)
        {
            triggerDataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modSubRgnChormaAssistType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modSubRgnChormaAssistType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(chromaDiffThreshold);
    PARAMETER_INITIALIZE(AECFaceROIUpdate);
    PARAMETER_INITIALIZE(referenceIlluminant);

    result = chromaDiffThreshold.Load(entry, alignment);
    if (result)
    {
        result = AECFaceROIUpdate.Load(entry, alignment);
    }
    if (result)
    {
        result = referenceIlluminant.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modSubRgnChormaAssistType_AECFaceROIUpdateStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modSubRgnChormaAssistType::AECFaceROIUpdateStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(hysteresisPerc);
    PARAMETER_INITIALIZE(hysteresisFrameCnt);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hysteresisPerc, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, hysteresisFrameCnt, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modSubRgnChormaAssistType_referenceIlluminantStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modSubRgnChormaAssistType::referenceIlluminantStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(useCase);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    if (result)
    {
        result = modawbfaceassistv1::ReadEnum_refIlluSubRgnUseCaseType(entry, useCase, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modContourFilterType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modContourFilterType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load modFaceDecisionTree
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool modFaceDecisionTree::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(ROISizeRatioThreshold);
    PARAMETER_INITIALIZE(eyeAngleDiffThreshold);
    PARAMETER_INITIALIZE(faceAngleDiffThreshold);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, ROISizeRatioThreshold, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, eyeAngleDiffThreshold, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, faceAngleDiffThreshold, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatixAWBFaceAssistV1Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixAWBFaceAssistV1Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(enableFlag);
    PARAMETER_INITIALIZE(faceAssistSAWeight);
    PARAMETER_INITIALIZE(FaceROICtrl);
    PARAMETER_INITIALIZE(faceDetectCtrl);
    PARAMETER_INITIALIZE(ConvSpeed);
    PARAMETER_INITIALIZE(darkThresholdData);
    PARAMETER_INITIALIZE(SatThresholdData);
    PARAMETER_INITIALIZE(confidenceLevel);
    PARAMETER_INITIALIZE(darkSkinTargetPt);
    PARAMETER_INITIALIZE(lightSkinTargetPt);
    PARAMETER_INITIALIZE(skinTypeInterpWeight);
    PARAMETER_INITIALIZE(AWBGainLockCtrl);
    PARAMETER_INITIALIZE(CbCrHighThreshold);
    PARAMETER_INITIALIZE(CbCrLowThreshold);
    PARAMETER_INITIALIZE(SkinStatsScreening);
    PARAMETER_INITIALIZE(RefIlluFace);
    PARAMETER_INITIALIZE(SubRegionChormaAssist);
    PARAMETER_INITIALIZE(ContourFilter);
    PARAMETER_INITIALIZE(FaceDecisionTree);

    result = moduleID.Load(entry, alignment);
    if (result)
    {
        result = module_version.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableFlag, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, faceAssistSAWeight, alignment)
    if (result)
    {
        result = FaceROICtrl.Load(entry, alignment);
    }
    if (result)
    {
        result = faceDetectCtrl.Load(entry, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, ConvSpeed, alignment)
    if (result)
    {
        result = darkThresholdData.Load(entry, alignment);
    }
    if (result)
    {
        result = SatThresholdData.Load(entry, alignment);
    }

    // Reading Array confidenceLevel from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        confidenceLevelCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, confidenceLevelCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            confidenceLevelID = pointerEntry->ID;
            DupObjInfo* confidenceLevelDupInfo = pointerEntry->GetDupObject(confidenceLevelCount, file, line);
            if (confidenceLevelDupInfo != nullptr && confidenceLevelDupInfo->Object != nullptr)
            {
                confidenceLevel = (modFaceAssistV1ConfidenceDataType*)confidenceLevelDupInfo->Object;
                if (confidenceLevel == nullptr)
                {
                    if (confidenceLevelDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "confidenceLevel duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            confidenceLevelDupInfo->File, confidenceLevelDupInfo->Line, confidenceLevelDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "confidenceLevel duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            confidenceLevelDupInfo->Line, confidenceLevelDupInfo->RefCount);
                    }
                }
            }

            if (confidenceLevel == nullptr)
            {
                if (result && confidenceLevelCount > 0)
                {
                    confidenceLevel = PARAMETER_NEW modFaceAssistV1ConfidenceDataType[confidenceLevelCount];
                    (void)pointerEntry->SetDupObject(confidenceLevel, confidenceLevelCount, file, line);
                }
                if (!result || confidenceLevel == nullptr)
                {
                    if (confidenceLevelCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate confidenceLevel\n");
                    }
                    confidenceLevelCount = 0;
                }
                for (UINT32 i = 0; result && i < confidenceLevelCount; i++)
                {
                    result = confidenceLevel[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (confidenceLevel == nullptr)
        {
            confidenceLevelCount = 0;
        }
    }
    if (result)
    {
        result = darkSkinTargetPt.Load(entry, alignment);
    }
    if (result)
    {
        result = lightSkinTargetPt.Load(entry, alignment);
    }

    // Reading Array skinTypeInterpWeight from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        skinTypeInterpWeightCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, skinTypeInterpWeightCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            skinTypeInterpWeightID = pointerEntry->ID;
            DupObjInfo* skinTypeInterpWeightDupInfo = pointerEntry->GetDupObject(skinTypeInterpWeightCount, file, line);
            if (skinTypeInterpWeightDupInfo != nullptr && skinTypeInterpWeightDupInfo->Object != nullptr)
            {
                skinTypeInterpWeight = (modFaceAssistV1ConfidenceDataType*)skinTypeInterpWeightDupInfo->Object;
                if (skinTypeInterpWeight == nullptr)
                {
                    if (skinTypeInterpWeightDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "skinTypeInterpWeight duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            skinTypeInterpWeightDupInfo->File, skinTypeInterpWeightDupInfo->Line, skinTypeInterpWeightDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "skinTypeInterpWeight duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            skinTypeInterpWeightDupInfo->Line, skinTypeInterpWeightDupInfo->RefCount);
                    }
                }
            }

            if (skinTypeInterpWeight == nullptr)
            {
                if (result && skinTypeInterpWeightCount > 0)
                {
                    skinTypeInterpWeight = PARAMETER_NEW modFaceAssistV1ConfidenceDataType[skinTypeInterpWeightCount];
                    (void)pointerEntry->SetDupObject(skinTypeInterpWeight, skinTypeInterpWeightCount, file, line);
                }
                if (!result || skinTypeInterpWeight == nullptr)
                {
                    if (skinTypeInterpWeightCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate skinTypeInterpWeight\n");
                    }
                    skinTypeInterpWeightCount = 0;
                }
                for (UINT32 i = 0; result && i < skinTypeInterpWeightCount; i++)
                {
                    result = skinTypeInterpWeight[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (skinTypeInterpWeight == nullptr)
        {
            skinTypeInterpWeightCount = 0;
        }
    }
    if (result)
    {
        result = AWBGainLockCtrl.Load(entry, alignment);
    }
    if (result)
    {
        result = CbCrHighThreshold.Load(entry, alignment);
    }
    if (result)
    {
        result = CbCrLowThreshold.Load(entry, alignment);
    }
    if (result)
    {
        result = SkinStatsScreening.Load(entry, alignment);
    }
    if (result)
    {
        result = RefIlluFace.Load(entry, alignment);
    }
    if (result)
    {
        result = SubRegionChormaAssist.Load(entry, alignment);
    }
    if (result)
    {
        result = ContourFilter.Load(entry, alignment);
    }
    if (result)
    {
        result = FaceDecisionTree.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixAWBFaceAssistV1Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1TargetDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1TargetDataType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1ConfidenceDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1ConfidenceDataType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1StatThresholdDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1StatThresholdDataType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1CbCrDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1CbCrDataType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1CbCrDiffDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1CbCrDiffDataType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1OutlierDistType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1OutlierDistType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceAssistV1SkinStatsScreeningType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceAssistV1SkinStatsScreeningType::Unload(ParameterFileSymbolTable* const __table)
{
    StatsOutlierDistanceScreening.Unload(__table);
    lockDecisionPointOutlierDistanceScreening.Unload(__table);
    CrCbDiffForChromaFilter.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modFaceSARefIlluConfidenceDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modFaceSARefIlluConfidenceDataType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modReferenceCCTType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modReferenceCCTType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(CCT))
    {
        PARAMETER_DELETE[] CCT;
    }
    CCT = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modRefIlluCtrlType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modRefIlluCtrlType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < referenceCCTsCount; i++)
    {
        referenceCCTs[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(referenceCCTs))
    {
        PARAMETER_DELETE[] referenceCCTs;
    }
    referenceCCTs = nullptr;
    for (UINT32 i = 0; i < refIlluConfidenceLevelCount; i++)
    {
        refIlluConfidenceLevel[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(refIlluConfidenceLevel))
    {
        PARAMETER_DELETE[] refIlluConfidenceLevel;
    }
    refIlluConfidenceLevel = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modSubRgnChromaDiffThDataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modSubRgnChromaDiffThDataType::Unload(ParameterFileSymbolTable* const __table)
{
    for (UINT32 i = 0; i < triggerDataCount; i++)
    {
        triggerData[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(triggerData))
    {
        PARAMETER_DELETE[] triggerData;
    }
    triggerData = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload modSubRgnChormaAssistType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void modSubRgnChormaAssistType::Unload(ParameterFileSymbolTable* const __table)
{
    chromaDiffThreshold.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatixAWBFaceAssistV1Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixAWBFaceAssistV1Type::Unload(ParameterFileSymbolTable* const __table)
{
    darkThresholdData.Unload(__table);
    SatThresholdData.Unload(__table);
    for (UINT32 i = 0; i < confidenceLevelCount; i++)
    {
        confidenceLevel[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(confidenceLevel))
    {
        PARAMETER_DELETE[] confidenceLevel;
    }
    confidenceLevel = nullptr;
    darkSkinTargetPt.Unload(__table);
    lightSkinTargetPt.Unload(__table);
    for (UINT32 i = 0; i < skinTypeInterpWeightCount; i++)
    {
        skinTypeInterpWeight[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(skinTypeInterpWeight))
    {
        PARAMETER_DELETE[] skinTypeInterpWeight;
    }
    skinTypeInterpWeight = nullptr;
    CbCrHighThreshold.Unload(__table);
    CbCrLowThreshold.Unload(__table);
    SkinStatsScreening.Unload(__table);
    RefIlluFace.Unload(__table);
    SubRegionChormaAssist.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixAWBFaceAssistV1Type::Release()
{
    ((chromatixAWBFaceAssistV1TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatixAWBFaceAssistV1Type::GetModified()
{
    return ((chromatixAWBFaceAssistV1TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatixAWBFaceAssistV1TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatixAWBFaceAssistV1TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatixAWBFaceAssistV1TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatixAWBFaceAssistV1TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatixAWBFaceAssistV1TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatixAWBFaceAssistV1TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatixAWBFaceAssistV1TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatixAWBFaceAssistV1TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatixAWBFaceAssistV1TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixAWBFaceAssistV1TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatixAWBFaceAssistV1TypeClass::Release()
{
    ParameterModule::Release();
}

MODAWBFACEASSISTV1_NAMESPACE_END
